********************************
*  MICROPAC HC11 DEMO PROGRAM
*  VERSION 1.2
*  COPYRIGHT 1996
*  EMAC, INC.
********************************

*
* Since the Motorola assembler doesn't allow conditional assembly, 
* you will notice that sections of code that aren't used are JMPed 
* over.  You can also cut out the unused portions if you want to, 
* but make a backup copy of this file for debugging purposes.
* There must be a jumper in position C of JP7 in order for the keypad 
* routine to work.
*
* When using BUFFALO monitor, type GO 400 after the program has
* been downloaded.  Also make sure JP10 is in the SS position
* to enable single stepping.
*
* CONVERSION TO ROM RESIDENT VERSION
* When converting the program to a ROM resident version remove the JMP
* that skips over the initialization code and change the ORG $400
* to ORG $9000.  The S-record file can then be loaded into an EPROM
* Programmer to put the code in EPROM.  The new EPROM will replace the 
* BUFFALO monitor.
*
* See more comments below.
*

IRAMSZ	EQU	$3FF 		size of internal ram
IRAM  	EQU	$0000		internal RAM pointer
IOADDR	EQU	$8000		I/O ADDR BASE

JMPOP	EQU	$7E		OPCODE FOR JMP


* THIS IS SET TO 0 NOW BECAUSE X IS USED TO OFFSET THEM
REGADDR  EQU	$0000		START OF INTERNAL REGISTERS

PORTA	EQU	REGADDR
DDRA     EQU	REGADDR+$01
PORTG    EQU	REGADDR+$02
DDRG     EQU	REGADDR+$03
PORTB    EQU	REGADDR+$04
PORTF    EQU	REGADDR+$05
PORTC    EQU	REGADDR+$06
DDRC     EQU	REGADDR+$07
PORTD    EQU	REGADDR+$08
DDRD     EQU	REGADDR+$09
PORTE    EQU	REGADDR+$0A
CFORC    EQU	REGADDR+$0B
OC1M     EQU	REGADDR+$0C
OC1D     EQU	REGADDR+$0D
TCNTHI   EQU	REGADDR+$0E
TCNTLO   EQU	REGADDR+$0F

TIC1HI   EQU	REGADDR+$10
TIC1LO   EQU	REGADDR+$11
TIC2HI   EQU	REGADDR+$12
TIC2LO   EQU	REGADDR+$13
TIC3HI   EQU	REGADDR+$14
TIC3LO   EQU	REGADDR+$15
TOC1HI   EQU	REGADDR+$16
TOC1LO   EQU	REGADDR+$17
TOC2HI   EQU	REGADDR+$18
TOC2LO   EQU	REGADDR+$19
TOC3HI   EQU	REGADDR+$1A
TOC3LO   EQU	REGADDR+$1B
TOC4HI   EQU	REGADDR+$1C
TOC4LO   EQU	REGADDR+$1D
TI4O5HI  EQU	REGADDR+$1E
TI4O5LO  EQU	REGADDR+$1F

TCTL1    EQU	REGADDR+$20
TCTL2    EQU	REGADDR+$21
TMSK1    EQU	REGADDR+$22
TFLG1    EQU	REGADDR+$23
TMSK2    EQU	REGADDR+$24
TFLG2    EQU	REGADDR+$25
PACTL    EQU	REGADDR+$26
PACNT    EQU	REGADDR+$27
SPCR     EQU	REGADDR+$28
SPSR     EQU	REGADDR+$29
SPDR     EQU	REGADDR+$2A
BAUD     EQU	REGADDR+$2B
SCCR1    EQU	REGADDR+$2C
SCCR2    EQU	REGADDR+$2D
SCSR     EQU	REGADDR+$2E
SCDR     EQU	REGADDR+$2F
ADCTL    EQU	REGADDR+$30
ADR1     EQU	REGADDR+$31
ADR2     EQU	REGADDR+$32
ADR3     EQU	REGADDR+$33
ADR4     EQU	REGADDR+$34
BPROT    EQU	REGADDR+$35

OPT2     EQU	REGADDR+$38
OPTION   EQU	REGADDR+$39
COPRST   EQU	REGADDR+$3A
PPROG    EQU	REGADDR+$3B
HPRIO    EQU	REGADDR+$3C
INIT     EQU	REGADDR+$3D
TEST1    EQU	REGADDR+$3E
CONFIG   EQU	REGADDR+$3F

CSSTRH   EQU	REGADDR+$5C
CSCTL    EQU	REGADDR+$5D
CSGADR   EQU	REGADDR+$5E
CSGSIZ   EQU	REGADDR+$5F     

*==============================
*         EXTERNAL I/O
*==============================

* LOAD REG. X WITH THE CONSTANT FOR THE DESIRED CHIP SELECT
CSIO1	EQU	IOADDR+$0060	START OF CHIP SELECT 1
CSIO2	EQU	IOADDR+$0800	START OF CHIP SELECT 2

* ROMWR MUST POINT TO AN ADDRESS WHERE ROM IS SELECTED (I.E. THE ADDRESS
* MUST NOT BE USED BY INTERNAL DEVICES OR EXT. CHIP SELECTS)
ROMWR	EQU	$E000	ROM WRITE CHIP SELECT
OBCS	EQU	CSIO1	ON BOARD DEVICE CHIP SELECT
EXSEL	EQU	CSIO2	EXPANSION HEADER CHIP SELECT

*
*	THE FOLLOWING ASSUME X POINTS TO ROMWR	
DLATCH	EQU	0	DATA LATCH 
DAC0	EQU	4	4 CHANNELS XXX4 TO XXX7

*
*	THE FOLLOWING ASSUME X POINTS TO OBCS
LCDCMD	EQU	0	COMMAND REG
LCDDAT	EQU	1	DATA REG
*	SCN2651 PORTS	
SERDR	EQU	4	WR TO TRANSMIT, RD TO RECEIVE
SERCNST	EQU	5	RD STATUS, WR TO SYN1/2/DLE REGS
SERMODE	EQU	6	RD/WR MODE REGS 1/2
SERCMD	EQU	7	RD/WR COMMAND REGISTER
*	KEYPAD INTERFACE
KEYPRT	EQU	8



*
* INTERRUPT VECTORS FOR COMPATIBILITY WITH BUFFALO MONITOR
*
	ORG	$C4	
JSCI 	RMB	3
JSPI 	RMB	3
JPAIE	RMB	3
JPAO 	RMB	3
JTOF 	RMB	3
JTOC5	RMB	3
JTOC4	RMB	3
JTOC3	RMB	3
JTOC2	RMB	3
JTOC1	RMB	3
JTIC3	RMB	3
JTIC2	RMB	3
JTIC1	RMB	3
JRTI 	RMB	3
JIRQ 	RMB	3
JXIRQ	RMB	3
JSWI 	RMB	3
JILLOP	RMB	3
JCOP    	RMB	3
JCLM    	RMB	3


***************************************************************************
*		           INTERNAL RAM
***************************************************************************
	ORG	$100
ESCFLAG	RMB	1	SET TO 1 IF ESC RECEIVED BY LCD DRIVER	
SLATCH	RMB	1	RAM SHADOW OF DATA LATCH
DKEYPD	RMB	1	KEYPAD DATA
FKEYPD	RMB	1	KEYPAD DATA AVAILABLE FLAG
CONSTAT	RMB	3	THESE ARE 3 BYTE JMP VECTORS IN THE 0 PAGE
INCONNE	RMB	3
IOBaseV	RMB	2	VECTOR WHERE IOBASE IS STORED
XONCH  	RMB	1	ADDRESS TO STORE XON CHAR
XOFFCH 	RMB	1
       
***************************************************************************
*		        MICROPAC HC11 CODE
***************************************************************************
	ORG	$400  	DEFAULT TO ABOVE STACK IF RESIDING IN RAM
         
STARTUP  EQU      *

         JMP	SKIP		* SKIP INITS IF LOADING INTO BUFFALO MONITOR

*        a/d must settle for 100ms after this
	LDAA	#%10100011	see pg 6-4 of MC68HC11F1 TECHNICAL DATA manual
	STAA	$1039	 	configuration options
	LDAA	#$08 	 	move regs to $8000, internal RAM to $0000
	STAA	$103D		store in init reg.

	LDX 	IOBase		ROM CONSTANT FOR IO BASE ADDRESS
        
	LDAA	#%01000111	CSIO1 valid during E. CSIO2 valid on addr.
	STAA	CSGSIZ,X      

*        enable CSIO1, CSIO2, make them active low and enable PCSEN for 64k
	LDAA	#%10100100
	STAA	CSCTL,X

*	enable clk stretching ON CSIO1 & 2
 	LDAA	#%11110000
 	STAA	CSSTRH,X

	NOP			NOPs fix assembler s19 bug
	NOP
	NOP	
	NOP
	NOP
	NOP	
	NOP
	NOP
	NOP			

* 	PG 0,1 ARE USED TO SELECT 1 OF 4 32K PAGES OF A 128K RAM.
*	PG 2,3 ARE HANDSHAKE LINES	
	LDAA	#%00001011	PG 0,1,3 ARE OUTPUTS
	STAA	DDRG,X 
	CLRA			CLEAR A	
	STAA	PORTG,X		SEND ALL 0

	LDAA	#$FF           	code to config. port a for output
	STAA	DDRA,X         	port a direction reg

SKIP	
	LDS	#IRAM+IRAMSZ    	load SP with value of end of IRAM


*****************************
**      DEMO ROUTINES      **
*****************************
	LDX	#IOADDR
	STX	IOBaseV		THIS VECTOR MUST BE LOADED WITH THE CURRENT ADDRESS OF REGISTERS

	JSR	InitSCI
	JSR	INI2651		SEE IF SCN2651 INSTALLED
	BCS	DEMO2		SKIP IF DEVICE NOT INSTALLED
	JSR	COM1TST		DEMO THE SCN2651 IF INSTALLED
     	BRA	DEMO1		SKIP SCITST IF INSTALLED
DEMO2	JSR	SCITST		TEST THE STANDARD UART


DEMO1	JSR	DIGDEMO

***	NOTE THAT THE FOLLOWING ARE COMMENTED OUT SO THEY ARE NOT ASSEMBLED.
***  	THESE SHOULD ONLY BE ASSEMBLED IF THE OPTIONS ARE POPULATED.

**	JSR	KEYTST		DEMO THE KEYPAD
**	JSR	LCDTST		DEMO THE LCD DISPLAY
**	JSR	SCITST		TEST THE STANDARD UART
	
	SWI			RETURN TO BUFFALO	






*
* OUTPUT 00 TO FF TO DIGITAL I/O HEADER PINS PX12 TO PX19
*
DIGDEMO:
	CLRB			SELECT SCI PORT FOR DISPLAY
	LDX	#MSG2
	JSR	PSTRING


         LDAA	#0		
LOOP1    INCA
	BEQ 	LOOP3
         STAA	ROMWR+DLATCH

	LDY 	#0
LOOP2    INY                    	DELAY
         CPY	#500
         BNE	LOOP2
         BRA	LOOP1          	test loop          
LOOP3
	CLRB			SELECT SCI PORT FOR DISPLAY
	LDX	#MSG3
	JSR	PSTRING
	RTS

*
* GET 10 CHARACTERS FROM THE KEYPAD AND SEND THEM TO THE MAIN SERIAL PORT.
*

KEYTST:
	CLRB			SELECT SCI PORT FOR DISPLAY
	LDX	#MSG4
	JSR	PSTRING

	JSR	INITKEY		INIT THE KEYPAD ROUTINE
	LDAB	#10
KLP1
	JSR	KEYIN
	JSR	SCIOUT
	DECB
	BNE	KLP1
	RTS

*
* Send a message to the optional serial port (SCN2651) and echo the
* chars received back out, until CR is received.
*
COM1TST
	LDAB	#1		SELECT SCN2651 FOR DISPLAY
	LDX	#MSG5
	JSR	PSTRING

COM1T1	JSR	COM1IN
	CMPA	#13
	BEQ	COM1T2
	JSR	COM1OUT
	JSR	SCIOUT
	BRA	COM1T1
COM1T2	RTS		


*
* Send a message to the standard serial port (SCI) and echo the
* chars received back out, until CR is received.
*
SCITST
	CLRB			SELECT SCI PORT FOR DISPLAY (SEE PSTRING)
	LDX	#MSG1
	JSR	PSTRING

SCIECH	JSR	SCIINNE
	CMPA	#13		CARRIAGE RETURN?
	BEQ	SCIEC1		EXIT IF YES
	JSR	SCIOUT		ECHO THE CHAR
	BRA	SCIECH
SCIEC1	RTS





*
* Send a message to the 20*2 LCD
*
LCDTST:
	JSR	LCDINIT
	
	CLRB			SELECT SCI PORT FOR DISPLAY
	INCB
	LDX	#MSG6
	JSR	PSTRING

	LDAB	#2		SELECT LCD DISPLAY
	LDX	#MSG7
	JSR	PSTRING
	RTS

	rmb	10		TO FIX MOTOROLA ASSEMBLER BUG




MSG1	FCC	'MICROPAC HC11 demo program'
	FCB	10,10,13
	FCC	'The keys you press will be echoed until return is pressed'
	FCB	10,13,0

MSG2	FCB	10,10,13
	FCC	'0 TO $FF WILL BE SENT TO PX12 TO PX19.......'
	FCB	10,13,0

MSG3	FCC	'FINISHED'
	FCB	10,13,0

MSG4	FCB	10,10,13
	FCC	'Press 10 keys on the keypad'
	FCB	10,13,0

MSG5	FCB	10,10,13
	FCC	'SCN2651 serial port demo'
	FCB	10,13
	FCC	'The characters received on the SCN2651 port will be echoed'	
	FCB	10,13
	FCC	'to both serial ports until a carriage return is received.'
	FCB	10,13,0

MSG6	FCB	10,10,13
	FCC	'Look at the LCD'
	FCB	10,13,0


MSG7	FCB	26,10,13		clear display, and move to line 2
	FCC	'This is line 2'
	FCB	10,13			move to line 1
	FCC	'This is line 1'
	FCB	0





	RMB	10

*************************************************************************

*
* INIT KEYPAD:	There must be a jumper in position C (only) of JP7 in order
* 		for the keypad to cause an IRQ.
*
INITKEY	LDAA #$7E        	put "jmp IRQVEC" in jump table
        STAA JIRQ
        LDX  #IRQVEC
        STX  JIRQ+1     
 
	TPA
	ANDA	#%10101111
	TAP			ENABLE IRQ (KEYPAD) INTERRUPTS
	RTS


*
* KEYPAD INTERRUPT HANDLER
*
IRQVEC
	LDX	#OBCS
	LDAA	KEYPRT,X
	ANDA	#%00011111	
	ADDA	#'A'
	STAA	DKEYPD		STORE DATA IN RAM
	LDAA	#1
	STAA	FKEYPD		SET KEY READY FLAG
	RTI

*
* POLL THE KEYPAD VARIABLES FOR A KEY ENTRY
*
KEYIN	LDAA	FKEYPD
	BEQ	KEYIN	LOOP TILL KEY READY
	CLR	FKEYPD
	LDAA	DKEYPD	GET KEY
	RTS


	

	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP
	NOP





*************************************************************************
	









************************************************************************
*		  LCD DRIVERS FOR MICROPAC HC11
************************************************************************

INITDATA FCB       $38,08,01,06,$0e,$80,0
         
*        
* LCDOUT  Output the char in ACC to LCD        
* Following are some codes that are supported by this driver (20*2 displays)
*
*        0ah      move cursor to other line (i.e. from 1-2 or 2-1)
*        0dh      move cursor to beginning of line 
*        1ah      clear display and move cursor to home
*                  
*        
LCDOUT   
	PSHX
	LDX 	#OBCS   


lcdnt5   BITA	#%11100000  	SEE IF ANY OF UPPER 3 BITS SET
        	BNE 	REG1OUT       	IF NOT 0, YES, PRINT IT
        	BITA 	#%11111000	SEE IF CHAR IS < 7
        	BEQ 	REG1OUT       	IF LESS, Z=1 SO PRINT USER DEF CHAR 0-7

*    	SEE IF CONTROL CHAR
     	CMPA	#$0D	  	SEE IF CR
     	BNE 	LCNT1      	IF NOT CR, SKIP
     	LDAA	LCDCMD,X		READ COMMAND PORT TO FIND CURSOR POS
       	ORAA	#$80           	SET BIT 7 FOR DDRAM ADDR
       	ANDA	#%11100000     	MOVE TO LEFT (ONLY VALID ON 2 LINE DISPL)
       	BRA 	REG0OUT
        
LCNT1  	CMPA	#$0A		SEE IF LF
     	BNE 	LCNT2     	IF NOT LF, SKIP
     	LDAA	LCDCMD,X       	READ COMMAND PORT TO FIND CURSOR POS
       	EORA	#%01000000     	SWITCH LINE (ONLY VALID ON 2 LINE DISPL)
     	ORAA	#%10000000	SET BIT 7 FOR DDRAM ADDR
       	BRA 	REG0OUT
     			       
LCNT2  	CMPA	#$1B		IS IT ESC?
     	BNE	LCNT3      	IF NOT ESC, SKIP
       	RTS

LCNT3  	CMPA	#$1A		CLEAR SCREEN?
     	BNE	LCNT4      	EXIT IF NOT CLEAR SCREEN
     	LDAA	#1		CLEAR COMMAND
       	BRA	REG0OUT

*	OUTPUT THE CHAR IN A TO REG 1 (data register)
REG1OUT BRSET	LCDCMD,X $80 * 	LOOP TILL BUSY FLAG IS LOW
        STAA	LCDDAT,X          OUTPUT IT
LCNT4   PULX
	RTS

         
*	OUTPUT THE CHAR IN A TO REG 0
REG0OUT:
	BRSET	LCDCMD,X $80 * 	LOOP TILL BUSY FLAG IS LOW
	STAA	LCDCMD,X	OUTPUT THE COMMAND
	PULX
	RTS
        
        
* 
* LCDINIT  Init the LCD
*         
LCDINIT          
	LDX	#OBCS
	CLR      ESCFLAG           indicate no esc found
	JSR      DLAYA             5MS DELAY
	JSR      DLAYA             5MS DELAY
	JSR      DLAYA             5MS DELAY
	JSR      DLAYA             5MS DELAY

	LDAA     #$30
	STAA	LCDCMD,X          OUT TO LCD COMMAND PORT                  
	JSR      DLAYA             5MS DELAY         
	STAA	LCDCMD,X          OUT TO LCD COMMAND PORT                  
	JSR      DLAYA             5MS DELAY         
	STAA	LCDCMD,X          OUT TO LCD COMMAND PORT                  
 
 	LDY	#INITDATA    	POINT TO INIT DATA
*	the last command should take no more than 40 uS. 
	LDAB	#12               for timeout of 12 * (11 cyc) * (e period)
LCDNIT2 	BRCLR	LCDCMD,X $80 LCDNIT1    * JMP IF BUSY FLAG IS LOW (6 cyc)
	DECB			(2 cyc)	
	BNE	LCDNIT2		loop till timeout (3 cyc)
	ldaa	#55
	staa	PORTA,x
	BRA	LCDEXIT		EXIT IF TIMEOUT

LCDNIT1 BRSET	LCDCMD,X $80 * 	LOOP TILL BUSY FLAG IS LOW
	LDAA	0,Y		GET BYTE FROM INIT TABLE
	BEQ      LCDEXIT           EXIT IF 0
	INY			POINT TO NEXT BYTE        
	STAA	LCDCMD,X

	BRA	LCDNIT1           LOOP
         
LCDEXIT  RTS         





*
* Check status of optional port COM1 (see if a char waiting).
* If Z=1, char ready
*
C1STAT	EQU	*
	PSHX	           	Save the index register.
	LDX	#OBCS
	PSHA               	SAVE THE A-REG.
	LDAA   	SERCNST,X  	GET THE STATUS.
	BITA   	#2         	CHECK TO SEE IF A CHARACTER HAS BEEN RECEIVED.
	PULA               	RESTORE STATUS.
	PULX	           	Restore X.
	RTS                	RETURN W/ STATUS.




*
* LOAD A WITH CHAR RECEIVED FROM COM1
*
COM1IN	EQU	*
	PSHX
	LDX	#OBCS
COM1IN1	LDAA	SERCNST,X	GET STATUS
	BITA	#2     		CHECK RXRDY
	BEQ	COM1IN1		LOOP TILL READY
	LDAA	SERDR,X		GET DATA
	PULX
	RTS


C1INECH	JSR	COM1IN	 GET CHAR AND FALL THROUGH TO ECHO

*
* TX CHAR IN A THROUGH COM1
*
COM1OUT	EQU	*
	PSHX	
	PSHA			SAVE CHAR
	LDX	#OBCS
C1OUT1	LDAA	SERCNST,X	GET STATUS
	BITA	#1     		CHECK TXRDY
	BEQ	C1OUT1 		LOOP TILL READY
	PULA
	STAA	SERDR,X		STORE DATA
	PULX
	RTS


*
*
* Initialize the 2651 for 9600 baud at 8 MHz Extal.
*             Return CY=1 if device doesn't exist.
**********
INI2651  EQU	*
	PSHX
	LDX	#OBCS
	LDAA	SERCMD,X		READ COMMAND REG
	ANDA	#%11111010	DISABLE RX AND TX
	STAA	SERCMD,X		WRITE COMMAND REG
	CMPA	SERCMD,X		SEE IF CORRECT
      	BNE	INSCC3		SKIP IF DATA NOT SAME

INSCC1	LDAA	#%01001110   	1 STOP, N,8,ASYNC *16
	STAA	SERMODE,X	WRITE MR1
	LDAA	#%00111110	INTERNAL CLK AT 9600
	STAA	SERMODE,X	WRITE MR2
*	NORM, ACTIVE RTS,RESET ERRS,NORM,RXEN,ACTIVE DTR,TXEN
	LDAA	#%00110111
	STAA	SERCMD,X		WRITE COMMAND REG
	CMPA	SERCMD,X		SEE IF CORRECT
	CLC			ASSUME DEVICE EXISTS 
      	BEQ	INSCC2		SKIP IF DATA READ BACK OKAY
INSCC3	SEC
INSCC2	PULX
	RTS





*
* DO A 100 MS DELAY WITH 2 MHZ E-CLOCK
*
DLAYH:
	PSHY
	TPA		PUT FLAGS IN A
	LDY	#28571	28571*7CYC*0.5uS = 100mS
	BRA	DLYLUP

*
* DO A 5 MS DELAY WITH 2 MHZ E-CLOCK
*
DLAYA:
	PSHY
	TPA		PUT FLAGS IN A
	LDY	#1500	1500*7CYC*0.5uS = 5.25mS

DLYLUP:	DEY	      	4 CYCLES
	BNE	DLYLUP	3 CYCLES
	TAP		RESTORE FLAGS. NOTE THAT THIS CLEARS X FLAG!
	PULY
	RTS




*************
* OUTPUT ACC A IN BINARY TO COM0 AS HEX
*************
HEX2	PSHA
	PSHA
	JSR 	OUTLHLF
	PULA
	JSR 	OUTRHLF
	PULA
	RTS

OUTLHLF  LSRA            	shift data to right
         LSRA
         LSRA
         LSRA
OUTRHLF  ANDA 	#$0F	mask top half
         ADDA 	#$30	convert to ascii
         CMPA 	#$39
         BLE  	OUTA	jump if 0-9
         ADDA 	#$07	convert to hex A-F
OUTA     JSR  	COM1OUT	output character
         RTS




*************************************************************************
	
*	ORG	$FF00

*					      *
*
SCIINNE  EQU    *
	PSHX		Save the index register.
	LDX	IOBaseV
SCIIN1   LDAA	SCSR,X	GET SCI STATUS.
         ANDA	#$20  	HAS A CHARACTER BEEN RECIEVED?
         BEQ 	SCIIN1	NO. WAIT FOR CHARACTER TO BE RECIEVED.
         LDAA	SCDR,X	GET THE CHARACTER.
	PULX 	      	Restore X.
         RTS            	RETURN.


SCIIN	BSR	SCIINNE	GET CHAR WITH NO ECHO AND FALL THROUGH TO ECHO
		 
*
*
SCIOUT   EQU    *
	PSHX	         Save the index register.
	LDX	IOBaseV
         PSHA         	SAVE THE CHARACTER TO SEND.

SCIOUT1  LDAA  	SCSR,X 	GET THE SCI STATUS.
         BITA  	#$80   	HAS THE LAST CHARACTER BEEN SHIFTED OUT?
         BEQ   	SCIOUT1	NO. WAIT TILL IT HAS.
         PULA           	RESTORE CHARACTER TO SEND.
         STAA  	SCDR,X   SEND THE CHARACTER.
	PULX	      	Restore X.
         RTS        	RETURN.
*
*
SCISTAT  EQU    *
	PSHX	      	Save the index register.
	LDX	IOBaseV
	PSHA           	SAVE THE A-REG.
	LDAA   	SCSR,X 	GET THE SCI STATUS.
	BITA   	#$20   	CHECK TO SEE IF A CHARACTER HAS BEEN RECIEVED.
	PULA           	RESTORE STATUS.
	PULX	       	Restore X.
	RTS            	RETURN W/ STATUS.

*
* Send the string pointed to by X and delimited by $00 to the device selected by B.
* B=0 :   Main SCI
* B=1 :   Optional serial port COM1
* B=2 :   LCD display
*
PSTRING
	LDAA	0,X
	BEQ	PSEXIT		EXIT IF $00 FOUND
	JSR	CHAROUT		OUTPUT TO THE DEVICE SELECTED
	INX			INC THE POINTER
	BRA	PSTRING
PSEXIT	INX			POINT TO CHAR AFTER $00
	RTS

*
* Output to the device selected by B.  This assumes that the device has been initialized.
*
CHAROUT:
	CMPB	#0
	BNE	CHRT1
	JSR	SCIOUT		SCI port
CHRT1	CMPB	#1
	BNE	CHRT2
	JSR	COM1OUT		SCN2651
CHRT2	CMPB	#2
	BNE	CHRTX
	JSR	LCDOUT		LCD display
CHRTX	RTS

*
*
IODevInit:
	LDX	#IOADDR
	STX	IOBaseV	
	JSR	LCDINIT
	JSR	InitSCI
	JSR	INI2651
	RTS

	RMB	10 		*FIX ASSEMBLER S19 BUG
*
*
InitSCI  EQU 	*
 	PSHX	     	Save the index register.
 	LDX 	IOBaseV
	LDAA	#$30   	SET BAUD RATE TO 9600.
	STAA	BAUD,X
	CLR 	SCCR1,X	SET FOR 8 BIT OPERATION, DISABLE WAKEUP.
	LDAA	#$0C   	ENABLE THE TRANSMITER & RECEIVER.
	STAA	SCCR2,X
	LDAA	#$11   	GET THE XON CHARACTER (CONTROL-Q).
	STAA	XONCH  	INITALIZE THE XON REGISTER.
	LDAA	#$13   	GET THE XOFF CHARACTER (CONTROL-S).
	STAA	XOFFCH 	INITALIZE THE XOFF CHARACTER.
 	PULX
	RTS             	RETURN.

*
*

	ttl	Config/Reset/Interrupt Vectors
	pag
	
*	 org	$ffa0
* INPUT VECTORS
IOVects	 
	fdb	C1INECH
	fdb	SCIIN
	fdb	0
	fdb	0
	fdb	0
	fdb	0
	fdb	0
	fdb	0

* OUTPUT VECTORS
	fdb	COM1OUT
	fdb	SCIOUT
	fdb	LCDOUT
	fdb	0
	fdb	0
	fdb	0
	fdb	0
	fdb	0
*
*
*       	org	$ffc0
RAMStart	fdb	$4000		starting address of system RAM.
RAMSize 	fdb	$2000		size of BASIC11 RAM Buffer.
EEStart 	fdb	$6000		starting address of program storage EEPROM 
EESize  	fdb	$1000		size of the program storage EEPROM
IOBase  	fdb	IOADDR		Base Address of the I/O Registers
TimeVal 	fdb	62500		value used for generating 'Time' Interrupt
UserInit	fdb	IODevInit	Used to initialize console/other hardware.
DFLOPADR	fdb	$FFFF		; Address of flip-flop used to connect the HC11 SCI
*					; to the host port connector.

 
* INTERRUPT VECTOR TABLE
	ORG	$FFD6			      
	FDB	$00C4		SCI SERIAL SYSTEM.
	FDB	$00C7		SPI TRANSFER COMPLETE
	FDB	$00CA		PULSE ACCUMULATOR INPUT EDGE
	FDB	$00CD		PULSE ACCUMULATOR OVERFLOW
	FDB	$00D0		TIMER OVERFLOW	
	FDB	$00D3  		TIMER OUTPUT COMPARE 5
	FDB	$00D6  		TIMER OUTPUT COMPARE 4
	FDB	$00D9 		TIMER OUTPUT COMPARE 3
	FDB	$00DC  		TIMER OUTPUT COMPARE 2
	FDB	$00DF  		TIMER OUTPUT COMPARE 1
	FDB	$00E2 		TIMER INPUT CAPTURE 3
	FDB	$00E5		TIMER INPUT CAPTURE 2
	FDB	$00E8		TIMER INPUT CAPTURE 1
	FDB	$00EB	 	REAL TIME INTERRUPT
	FDB	IRQVEC		$00EE 		IRQ INTERRUPT
	FDB	$00F1		XIRQ INTERRUPT.
	FDB	$00F4		SOFTWARE INTERRUPT
	FDB	$00F7	      	ILLEGAL OPCODE TRAP	
	FDB	$00FA	     	WATCH DOG TIMER FAIL (COP). 
	FDB	$00FD	    	CLOCK MONITOR FAIL.
	FDB	STARTUP 	RESET VECTOR
