*
* 6 8 H C 1 1   R E A L   T I M E   C L O C K   D R I V E R S
*
* COPYRIGHT 1995, EMAC INC.
* 
*
* Due to the nature of access of the real time clock (RTC) the code must 
* NEVER reside in EXTERNAL RAM, so when using the buffalo monitor to
* test your code, you must put the RTC drivers in the 6811's INTERNAL RAM.
*
* Even when NOT using the buffalo monitor 8 bytes must be allocated in
* the 6811'S INTERNAL RAM to store the RTC data.  The label is CLKREG.  
* Also, 1 byte of scratchpad RAM (named CLKDUM) must be in the EXTERNAL RAM.
*
* NOTE THAT INTERRUPTS MUST BE DISABLED DURING READ OR WRITE OF RTC
*
* The CLKREG ARRAY IS ORGANIZED AS FOLLOWS:
*CLKREG     = HUNDREDTHS OF SECS 	00-99
*CLKREG + 1 = SECONDS		00-59
*CLKREG + 2 = MINUTES		00-59
*CLKREG + 3 = HOURS    		00-23
*CLKREG + 4 = DAY OF WEEK		00-07
*CLKREG + 5 = DAY OF MONTH		01-31
*CLKREG + 6 = MONTH    		01-12
*CLKREG + 7 = YEAR     		00-99
	   
*	   
* WRSCL= COPY THE 8 BYTES OF DATA TO THE RTC REGISTERS.  NOTE THAT
*   THE ROUTINE MODIFIES THE DAY OF WEEK REGISTER IN CLKREG.
*
WRSCL	EQU	*
	LDX 	#CLKDUM
	LDAB	#65		* DO 65 BIT READ IN CASE IN MIDDLE OF ANOTHER
	   			* INCOMPLETE READ
WRSCS	LDAA	0,X		* READ DUMMY REG
	DECB	
	BNE	WRSCS		* LOOP TILL 0

	TPA			* SAVE INTERRUPT STATUS
	PSHA
	SEI			* DISABLE INTS

        * SEND PATTERN RECOGNITION SEQUENCE
	LDX	#SWCPAT		* TABLE OF SMARTCLOCK PATTERN
WRCS3	LDAA	0,X		* GET PATTERN BYTE
	BEQ	WRCS2		* SKIP IF END MARKER FOUND
	LDAB	#8		* NUMBER OF BITS TO SEND
WRCS1	STAA	CLKDUM
	LSRA			* SHIFT RIGHT
	DECB
	BNE	WRCS1		* SEND ALL 8 BITS
	INX			* PT TO NEXT PATTERN BYTE
	BRA	WRCS3

	* NOW STORE DATA FROM CLKREG INTO THE CLOCK
	
WRCS2	LDX	#CLKREG
	LDAA	4,X		* GET DAY
	ANDA	#%00000111	* MASK ALL BUT DAY (ENABLE OSC)
	ORAA	#%00010111	* DISABLE RESET PIN
	STAA	4,X
	
WRCS5	LDAB	#8		* NUMBER OF BITS
	LDAA	0,X		* READ CLKREG REGISTER
WRCS4	STAA	CLKDUM		* STORE IN DUMMY REG
	LSRA			* SHIFT IT
	DECB	
	BNE	WRCS4		* LOOP TILL ALL BITS SHIFTED TO BIT 0
	INX			* SELECT NEXT REGISTER
	CPX	#CLKREG+8	* HAVE WE DONE ALL REGS?
	BNE	WRCS5		* GET NEXT REG IF NOT
	PULA		
	TAP			* RESTORE INT STATUS
	RTS


*
* RDSCL= COPY THE 8 RTC REGISTERS TO THE EIGHT BYTES STARTING AT CLKREG
*
RDSCL	EQU	*
	LDX	#CLKDUM
	LDAB	#65		* DO 65 BIT READ IN CASE IN MIDDLE OF ANOTHER
				* INCOMPLETE READ
RDSCS	LDAA	0,X		* READ DUMMY REG
	DECB	
	BNE	RDSCS		* LOOP TILL 0

	TPA			* SAVE INTERRUPT STATUS
	PSHA
	SEI			* DISABLE INTS

        * SEND PATTERN RECOGNITION SEQUENCE
	LDX	#SWCPAT		* TABLE OF WATCHDOG PATTERN
RDCS3	LDAA	0,X		* GET PATTERN BYTE
	BEQ	RDCS2		* SKIP IF END MARKER FOUND
	LDAB	#8		* NUMBER OF BITS TO SEND
RDCS1	STAA	CLKDUM
	LSRA			* SHIFT RIGHT
	DECB
	BNE	RDCS1		* SEND ALL 8 BITS
	INX			* PT TO NEXT PATTERN BYTE
	BRA	RDCS3

	* NOW READ THE SMARTWATCH
RDCS2	LDX	#CLKREG
RDCS5	LDAB	#8		* NUMBER OF BITS
RDCS4	LDAA	CLKDUM		* READ DUMMY REG
	LSRA			* PUT BIT 0 IN CY
	ROR	0,X		* SHIFT CARRY INTO CLKREG
	DECB	
	BNE	RDCS4		* LOOP TILL ALL BITS SHIFTED
	INX
	CPX	#CLKREG+8	* HAVE WE DONE ALL REGS?
	BNE	RDCS5		* GET NEXT REG IF NOT
	LDAA	CLKREG+4
	ANDA	#$07
	STAA	CLKREG+4
	PULA		
	TAP			* RESTORE INT STATUS
	RTS
		
SWCPAT	FCB	$C5,$3A,$A3,$5C,$C5,$3A,$A3,$5C,0

	
	
	



